
#include <LiquidCrystal_I2C.h>

// constants won't change
//for the LCD and thermistor
int Vo; // Holds the ADC Value
int Wo;
int Ws;
int To;
int Tamb;
int value_solar;
int solar_voltage;
int value_oven;
int oven_voltage;
int value_solar_current;
int solar_current;
int value_oven_current;
int oven_current;
int solar_watts;
int oven_watts;
int efficiency;
const float Beta = 3974.0;
const float roomTemp = 298.15;   // room temperature in Kelvin
const float Ro = 10000.0; // Resistance of the thermistor at roomTemp
const float R1 = 9710.0;  // Resistance of the known resistor
float R2, tKelvin, tCelsius, tFahrenheit;
float Rta; //calculated resistance of the ambient temerature sensor
float Rto; //calculated resistance of the oven temperature sensor 
int Tmax = 150; //maximum oven temp



//Pin definitions

//Input pins
const int oven_relay_pin_1 = 2;  // pin for oven relay
const int solar_current_pin = 36; //pin for solar current 
const int solar_volts_pin =   39; //pin for solar voltage 
const int oven_current_pin = 34; //pin for oven current 
const int oven_volts_pin =   35; //pin for oven current
const int oven_temp_pin = 27; //pin for oven temp  
const int ambient_temp_pin = 14; // pin for ambient temp 


//Output Pins
const int oven_relay_1_pin = 2;
const int I2C_SDA_Pin = 21;
const int I2C_SCL_Pin = 22;
const int neopixel1_data_pin = 12;
const int neopixel2_data_pin = 13;
const int solar_current_factor = 3.5;
const int oven_current_factor = 3.5;

  
LiquidCrystal_I2C lcd(0x27, 20, 4);



//for reading the solar panel voltage 
int value = 0;
float RD1 = 100000.0;//volt divider resistor 1
float RD2 = 33000.0;//volt divider resistor 2

//For reading the oven voltage
float RD3 = 1000000.0;
float RD4 = 33000.0;



void setup() {
  lcd.init();
  lcd.clear();
  lcd.backlight();      // Make sure backlight is on
  //initialize digital pin for the relay board as an output.
  pinMode (oven_relay_1_pin, OUTPUT);
  pinMode (neopixel1_data_pin, OUTPUT);
  pinMode (neopixel2_data_pin, OUTPUT);
 
}

void loop() {


  //read the oven thermistor and put the value on the LCD 
  Vo = analogRead(oven_temp_pin);
  Rto = R1 * (1023.0 / (float)Vo - 1.0); // Resistance of the Thermistor
  tKelvin = (Beta * roomTemp) /
            (Beta + (roomTemp * log(R2 / Ro)));
  To = tKelvin - 273.15;
  //ToF = (tCelsius * 9.0) / 5.0 + 32.0;

  lcd.setCursor(0, 0);
  lcd.print("To ");
  lcd.print(To);
  lcd.print(" F");
  delay(2000);


  //Calculate and display oven watts
  oven_watts = oven_current * oven_voltage;
  lcd.setCursor(11,0);
  lcd.print("Wo= ");
  lcd.print(oven_watts);
  delay(2000);
   

   //Turn the oven on/off 
  lcd.setCursor(18,0);
  if (To >= Tmax) {
  digitalWrite( oven_relay_1_pin,LOW);
  lcd.print("Off");
  }
  if  (To < Tmax) {
  digitalWrite(oven_relay_pin_1,HIGH);
  lcd.setCursor(18,0);
  lcd.print("On");
  delay(2000);
  }
     
  

  //read the ambient thermistor and put the value on the LCD 
  Vo = analogRead(ambient_temp_pin);
  Rta = R1 * (1023.0 / (float)Vo - 1.0); // Resistance of the Thermistor
  tKelvin = (Beta * roomTemp) /
            (Beta + (roomTemp * log(R2 / Ro)));
  Tamb = tKelvin - 273.15;
  //TambF = (tCelsius * 9.0) / 5.0 + 32.0;
  delay(2000);
  
  /*lcd.setCursor(10, 0);
  lcd.print("Tamb ");
  lcd.print(Tamb);
  lcd.print(" c");
  delay(500); */

//Print the Tmax, oven high safety temperature
  lcd.setCursor(0,1);
  lcd.print("Tmax ");
  lcd.print(Tmax);
  lcd.print(" C");
  delay(2000);
  
  


//Read and put the the solar panel voltage on the LCD 
 value_solar = analogRead(solar_volts_pin);
  solar_voltage = value_solar * (5.0/1024)*((RD1 + RD2)/RD2);
  lcd.setCursor(0,2);
  lcd.print("Vs = ");
  lcd.print(solar_voltage);
  delay(2000);
 
  
  //Read and display the solar current on the LCD 
  value_solar_current = analogRead(solar_current_pin);
  solar_current = value_solar_current * solar_current_factor;
  lcd.setCursor(8,2);
  lcd.print("Is= ");
  lcd.print(solar_current);
  delay(2000);


  //Calculate and display solar watts 
  solar_watts = solar_current * solar_voltage;
  lcd.setCursor(14,2);
  lcd.print("Ws = ");
  lcd.print(solar_watts);
  delay(2000);

  //read and put the oven voltage on the LCD
value_oven = analogRead(oven_volts_pin);
  oven_voltage = value_oven * (5.0/1024)*((RD3 + RD4)/RD4);
  lcd.setCursor(0,3);
  lcd.print("Vo = ");
  lcd.print(oven_voltage);
  delay(2000);

 //Read and display oven current on the LCD 
  value_oven_current = analogRead(oven_current_pin);
  oven_current = value_oven_current * oven_current_factor;
  lcd.setCursor(8,3);
  lcd.print("Io= ");
  lcd.print(oven_current);
  delay(2000);
  
  
  //Calculate and display efficiency of dc-dc converter 
  efficiency = Wo/Ws*100;
  lcd.setCursor(17,3);
  lcd.print("E = ");
  lcd.print(efficiency);

  delay(2000);


  //Animate the neopixel strings based on temp and solar input 
  
}
